VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsIStream"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 11/2017

' IUnknown interface
' https://docs.microsoft.com/en-us/windows/desktop/api/unknwn/nn-unknwn-iunknown

' ISequentialStream interface
' https://docs.microsoft.com/de-de/windows/desktop/api/objidl/nn-objidl-isequentialstream

' IStream interface
' https://docs.microsoft.com/en-us/windows/desktop/api/objidl/nn-objidl-istream

Option Explicit

' ----==== Const ====----
Private Const ERR_BASE As Long = 40670
Private Const CLSID_Stream As String = ""
Private Const IID_IStream As String = "{0000000C-0000-0000-C000-000000000046}"

Private Enum E_vtbOffsets
    '/*** IUnknown methods ***/
    vtb_QueryInterface
    vtb_AddRef
    vtb_Release
    '/*** ISequentialStream methods ***/
    vtb_Read
    vtb_Write
    '/*** IStream methods ***/
    vtb_Seek
    vtb_SetSize
    vtb_CopyTo
    vtb_Commit
    vtb_Revert
    vtb_LockRegion
    vtb_UnlockRegion
    vtb_Stat
    vtb_Clone
End Enum

' ----==== Variablen ====----
Private m_initialized As Boolean
Private m_Interface As Interface_Data

Public Function Read_Stream(ByRef pv() As Byte, ByVal cb As Long) As Long

    Dim lRet As Long
    Dim cbRead As Long
    Dim ppv As Long
    Dim pcbRead As Long
    
    ppv = VarPtr(pv(0))
    pcbRead = VarPtr(cbRead)
    
    If x_OleInvoke(vtb_Read, ppv, cb, pcbRead) Then
    
        lRet = cbRead
        
    End If
    
    Read_Stream = lRet
    
End Function

Public Function Write_Stream(ByRef pv() As Byte, ByVal cb As Long) As Long

    Dim lRet As Long
    Dim ppv As Long
    Dim cbWritten As Long
    Dim pcbWritten As Long
    
    ppv = VarPtr(pv(0))
    pcbWritten = VarPtr(cbWritten)
    
    If x_OleInvoke(vtb_Write, ppv, cb, pcbWritten) Then
    
        lRet = cbWritten
        
    End If
    
    Write_Stream = lRet
    
End Function

Public Function Seek_Stream(ByVal dlibMove As Currency, ByVal dwOrigin As _
    STREAM_SEEK) As Currency
    
    Dim cRet As Currency
    Dim libNewPosition As Currency
    Dim plibNewPosition As Long
    
    dlibMove = dlibMove / 10000
    plibNewPosition = VarPtr(libNewPosition)
    
    If x_OleInvoke(vtb_Seek, dlibMove, dwOrigin, plibNewPosition) Then
    
        cRet = libNewPosition * 10000
        
    End If
    
    Seek_Stream = cRet
    
End Function

Public Function SetSize(ByVal libNewSize As Currency) As Boolean

    Dim bRet As Boolean
    
    bRet = False
    libNewSize = libNewSize / 10000
    
    If x_OleInvoke(vtb_SetSize, libNewSize) Then
    
        bRet = True
        
    End If
    
    SetSize = bRet
    
End Function

Public Function CopyTo(ByVal pstm As Long, ByVal cb As Currency, ByRef _
    pcbRead As Currency, ByRef pcbWritten As Currency) As Boolean
    
    Dim bRet As Boolean
    Dim ppcbRead As Long
    Dim ppcbWritten As Long
    
    bRet = False
    cb = cb / 10000
    ppcbRead = VarPtr(pcbRead)
    ppcbWritten = VarPtr(pcbWritten)
    
    If x_OleInvoke(vtb_CopyTo, pstm, cb, ppcbRead, ppcbWritten) Then
    
        bRet = True
        
    End If
    
    CopyTo = bRet
    
End Function

Public Function Commit(Optional ByVal grfCommitFlags As STGC = STGC_DEFAULT) As _
    Boolean
    
    Dim bRet As Boolean
    
    bRet = False
    
    If x_OleInvoke(vtb_Commit, grfCommitFlags) Then
    
        bRet = True
        
    End If
    
    Commit = bRet
    
End Function

Public Function Revert() As Boolean

    Dim bRet As Boolean
    
    bRet = False
    
    If x_OleInvoke(vtb_Revert) Then
    
        bRet = True
        
    End If
    
    Revert = bRet
    
End Function

Public Function LockRegion(ByVal libOffset As Currency, ByVal cb As Currency, _
    ByVal dwLockType As LOCKTYPE) As Boolean
    
    Dim bRet As Boolean
    
    bRet = False
    cb = cb / 10000
    libOffset = libOffset / 10000
    
    If x_OleInvoke(vtb_LockRegion, libOffset, cb, dwLockType) Then
    
        bRet = True
        
    End If
    
    LockRegion = bRet
    
End Function

Public Function UnlockRegion(ByVal libOffset As Currency, ByVal cb As Currency, _
    ByVal dwLockType As LOCKTYPE) As Boolean
    
    Dim bRet As Boolean
    
    bRet = False
    cb = cb / 10000
    libOffset = libOffset / 10000
        
    If x_OleInvoke(vtb_UnlockRegion, libOffset, cb, dwLockType) Then
    
        bRet = True
        
    End If
    
    UnlockRegion = bRet
    
End Function

Friend Function Stat(ByRef pstatstg As STATSTG, Optional ByVal grfStatFlag As _
    STATFLAG = STATFLAG_DEFAULT) As Boolean
    
    Dim bRet As Boolean
    
    bRet = False
    
    If x_OleInvoke(vtb_Stat) Then
    
        bRet = True
        
    End If
    
    Stat = bRet
    
End Function

Public Function Clone() As Long

    Dim pRet As Long
    Dim ppstm As Long
    Dim pppstm As Long
    
    pppstm = VarPtr(ppstm)
    
    If x_OleInvoke(vtb_Clone, pppstm) Then
        
        pRet = ppstm
        
    End If
    
    Clone = pRet
    
End Function

' ----==== Helper Func ====----
Public Sub Initialize(ByVal pInterface As Long)

    m_Interface.owner = ObjPtr(Me)
    m_Interface.RaiseErrors = False
    m_Interface.ifc = pInterface
    m_initialized = True
    
End Sub

Private Sub Class_Terminate()

    Call ReleaseInterface(m_Interface)
    
End Sub

Public Sub RaiseError(Optional ByVal Raise As Boolean = False)
    
    m_Interface.RaiseErrors = Raise
    
End Sub

Private Function x_OleInvoke(ByVal vtb As E_vtbOffsets, ParamArray var()) As Boolean

    Dim ret As Long
    
    If Not m_initialized Then
    
        Call InterfaceError(m_Interface, ecd_OleInvoke)
        
    Else
    
        x_OleInvoke = oleInvoke(m_Interface, vtb, ret, True, var)
        
    End If
    
End Function

Public Sub x_RaiseError()

    With m_Interface
    
        If .ecd Then Err.Raise ERR_BASE + .ecd, App.EXEName & ".clsIStream", .etx
        
        .ecd = 0
        .etx = vbNullString
        
    End With
    
End Sub

